@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2012-2013 Aurimas Cernius
+ * Copyright (C) 2012-2013,2017 Aurimas Cernius
  * Copyright (C) 2009 Hubert Figuiere
  *
  * This program is free software: you can redistribute it and/or modify
@@ -19,8 +19,6 @@
  */
 
 
-#include <boost/format.hpp>
-
 #include <glibmm/i18n.h>
 
 #include <gtkmm/table.h>
@@ -54,8 +52,8 @@ namespace gnote {
       
       m_errorLabel.property_xalign() = 0;
       m_errorLabel.set_markup(
-        str(boost::format("<span foreground='red' style='italic'>%1%</span>")
-            % _("Name already taken")));
+        Glib::ustring::compose("<span foreground='red' style='italic'>%1</span>",
+            _("Name already taken")));
       
       table->attach (*label, 0, 1, 0, 1);
       table->attach (m_nameEntry, 1, 2, 0, 1);
@@ -78,13 +76,13 @@ namespace gnote {
     }
 
 
-    std::string CreateNotebookDialog::get_notebook_name()
+    Glib::ustring CreateNotebookDialog::get_notebook_name()
     {
       return sharp::string_trim(m_nameEntry.get_text());
     }
 
 
-    void CreateNotebookDialog::set_notebook_name(const std::string & value)
+    void CreateNotebookDialog::set_notebook_name(const Glib::ustring & value)
     {
       m_nameEntry.set_text(sharp::string_trim(value));
     }
