@@ -49,6 +49,7 @@ SUITE(SyncManagerTests)
     test::NoteManager *manager2;
     test::SyncManager *sync_manager1;
     test::SyncManager *sync_manager2;
+    std::list<Glib::ustring> files;
 
     Fixture()
     {
@@ -98,7 +99,7 @@ SUITE(SyncManagerTests)
       manager.create(title, make_note_content(title, body))->save();
     }
 
-    bool find_note(const std::list<Glib::ustring> & files, const Glib::ustring & title)
+    bool find_note_in_files(const Glib::ustring & title)
     {
       Glib::ustring content_search = Glib::ustring::compose("<note-title>%1</note-title>", title);
       for(auto file : files) {
@@ -110,76 +111,72 @@ SUITE(SyncManagerTests)
 
       return false;
     }
+
+    void get_notes_in_dir(const Glib::ustring & dir)
+    {
+      files.clear();
+      sharp::directory_get_files_with_ext(dir, ".note", files);
+    }
   };
 
+#define FIRST_SYNC(sync_manager, note_manager, manifest, client, ui) \
+  test::SyncClient::Ptr client = dynamic_pointer_cast<test::SyncClient>(sync_manager->get_client(manifest)); \
+  gnote::sync::SilentUI::Ptr ui = gnote::sync::SilentUI::create(*note_manager); \
+  sync_manager->perform_synchronization(ui);
+
+#define UPDATE_NOTE(manager, title, new_title, new_content) \
+  do { \
+    auto note = std::dynamic_pointer_cast<test::Note>(manager->find(title)); \
+    note->set_xml_content(make_note_content(new_title, new_title)); \
+    note->set_change_type(gnote::CONTENT_CHANGED); \
+    note->save(); \
+  } \
+  while(0)
+
   TEST_FIXTURE(Fixture, clean_sync)
   {
-    test::SyncClient::Ptr sync_client1 = dynamic_pointer_cast<test::SyncClient>(sync_manager1->get_client(manifest1));
-    gnote::sync::SilentUI::Ptr sync_ui = gnote::sync::SilentUI::create(*manager1);
-    sync_manager1->perform_synchronization(sync_ui);
+    FIRST_SYNC(sync_manager1, manager1, manifest1, sync_client, sync_ui)
 
     Glib::ustring syncednotesdir = syncdir + "/0/0";
     REQUIRE CHECK(sharp::directory_exists(syncednotesdir));
-    std::list<Glib::ustring> files;
-    sharp::directory_get_files_with_ext(syncednotesdir, ".note", files);
+    get_notes_in_dir(syncednotesdir);
     REQUIRE CHECK_EQUAL(3, files.size());
 
-    CHECK(find_note(files, "note1"));
-    CHECK(find_note(files, "note2"));
-    CHECK(find_note(files, "note3"));
+    CHECK(find_note_in_files("note1"));
+    CHECK(find_note_in_files("note2"));
+    CHECK(find_note_in_files("note3"));
   }
 
   TEST_FIXTURE(Fixture, first_sync_existing_store)
   {
-    // first sync
-    test::SyncClient::Ptr sync_client1 = dynamic_pointer_cast<test::SyncClient>(sync_manager1->get_client(manifest1));
-    gnote::sync::SilentUI::Ptr sync_ui1 = gnote::sync::SilentUI::create(*manager1);
-    sync_manager1->perform_synchronization(sync_ui1);
+    FIRST_SYNC(sync_manager1, manager1, manifest1, sync_client1, sync_ui1)
+    FIRST_SYNC(sync_manager2, manager2, manifest2, sync_client2, sync_ui2)
 
-    // sync from existing
-    test::SyncClient::Ptr sync_client2 = dynamic_pointer_cast<test::SyncClient>(sync_manager2->get_client(manifest2));
-    gnote::sync::SilentUI::Ptr sync_ui2 = gnote::sync::SilentUI::create(*manager2);
-    sync_manager2->perform_synchronization(sync_ui2);
-
-    std::list<Glib::ustring> files;
-    sharp::directory_get_files_with_ext(notesdir2, ".note", files);
+    get_notes_in_dir(notesdir2);
     REQUIRE CHECK_EQUAL(4, files.size()); // 3 downloaded notes + template
-    CHECK(find_note(files, "note1"));
-    CHECK(find_note(files, "note2"));
-    CHECK(find_note(files, "note3"));
+    CHECK(find_note_in_files("note1"));
+    CHECK(find_note_in_files("note2"));
+    CHECK(find_note_in_files("note3"));
   }
 
   TEST_FIXTURE(Fixture, merge_two_clients)
   {
-    // first sync
-    test::SyncClient::Ptr sync_client1 = dynamic_pointer_cast<test::SyncClient>(sync_manager1->get_client(manifest1));
-    gnote::sync::SilentUI::Ptr sync_ui1 = gnote::sync::SilentUI::create(*manager1);
-    sync_manager1->perform_synchronization(sync_ui1);
+    FIRST_SYNC(sync_manager1, manager1, manifest1, sync_client1, sync_ui1)
 
     create_note(*manager2, "note4", "content4");
-    test::SyncClient::Ptr sync_client2 = dynamic_pointer_cast<test::SyncClient>(sync_manager2->get_client(manifest2));
-    gnote::sync::SilentUI::Ptr sync_ui2 = gnote::sync::SilentUI::create(*manager2);
-    sync_manager2->perform_synchronization(sync_ui2);
+    FIRST_SYNC(sync_manager2, manager2, manifest2, sync_client2, sync_ui2)
 
     Glib::ustring syncednotesdir = syncdir + "/0/1";
     REQUIRE CHECK(sharp::directory_exists(syncednotesdir));
-    std::list<Glib::ustring> files;
-    sharp::directory_get_files_with_ext(syncednotesdir, ".note", files);
+    get_notes_in_dir(syncednotesdir);
     REQUIRE CHECK_EQUAL(2, files.size());  // note + template note
-    CHECK(find_note(files, "note4"));
+    CHECK(find_note_in_files("note4"));
   }
 
   TEST_FIXTURE(Fixture, download_new_notes_from_server)
   {
-    // first sync
-    test::SyncClient::Ptr sync_client1 = dynamic_pointer_cast<test::SyncClient>(sync_manager1->get_client(manifest1));
-    gnote::sync::SilentUI::Ptr sync_ui1 = gnote::sync::SilentUI::create(*manager1);
-    sync_manager1->perform_synchronization(sync_ui1);
-
-    // sync from existing
-    test::SyncClient::Ptr sync_client2 = dynamic_pointer_cast<test::SyncClient>(sync_manager2->get_client(manifest2));
-    gnote::sync::SilentUI::Ptr sync_ui2 = gnote::sync::SilentUI::create(*manager2);
-    sync_manager2->perform_synchronization(sync_ui2);
+    FIRST_SYNC(sync_manager1, manager1, manifest1, sync_client1, sync_ui1)
+    FIRST_SYNC(sync_manager2, manager2, manifest2, sync_client2, sync_ui2)
 
     // create new note and sync again
     create_note(*manager2, "note4", "content4");
@@ -188,50 +185,120 @@ SUITE(SyncManagerTests)
     // check sync dir contents
     Glib::ustring syncednotesdir = syncdir + "/0";
     REQUIRE CHECK(sharp::directory_exists(syncednotesdir));
-    std::list<Glib::ustring> files;
     sharp::directory_get_directories(syncednotesdir, files);
     CHECK_EQUAL(3, files.size());
 
     // sync to first client
     sync_manager1->perform_synchronization(sync_ui1);
-    files.clear();
-    sharp::directory_get_files_with_ext(notesdir1, ".note", files);
+    get_notes_in_dir(notesdir1);
     CHECK_EQUAL(5, files.size()); // 3 original + 1 from other client + template from other client
-    CHECK(find_note(files, "note4"));
+    CHECK(find_note_in_files("note4"));
   }
 
   TEST_FIXTURE(Fixture, upload_note_update)
   {
-    // first sync
-    test::SyncClient::Ptr sync_client1 = dynamic_pointer_cast<test::SyncClient>(sync_manager1->get_client(manifest1));
-    gnote::sync::SilentUI::Ptr sync_ui1 = gnote::sync::SilentUI::create(*manager1);
-    sync_manager1->perform_synchronization(sync_ui1);
+    FIRST_SYNC(sync_manager1, manager1, manifest1, sync_client1, sync_ui1)
 
     // update note and sync again
-    auto note = std::dynamic_pointer_cast<test::Note>(manager1->find("note2"));
-    note->set_xml_content(make_note_content("note4", "updated content"));
-    note->set_change_type(gnote::CONTENT_CHANGED);
-    note->save();
+    UPDATE_NOTE(manager1, "note2", "note4", "updated content");
     sync_client1->reparse();
     sync_manager1->perform_synchronization(sync_ui1);
 
     // check sync dir contents
     Glib::ustring syncednotesdir = syncdir + "/0";
     REQUIRE CHECK(sharp::directory_exists(syncednotesdir));
-    std::list<Glib::ustring> files;
     sharp::directory_get_directories(syncednotesdir, files);
     CHECK_EQUAL(2, files.size());
 
-    // sync from existing
-    test::SyncClient::Ptr sync_client2 = dynamic_pointer_cast<test::SyncClient>(sync_manager2->get_client(manifest2));
-    gnote::sync::SilentUI::Ptr sync_ui2 = gnote::sync::SilentUI::create(*manager2);
+    FIRST_SYNC(sync_manager2, manager2, manifest2, sync_client2, sync_ui2)
+
+    get_notes_in_dir(notesdir2);
+    REQUIRE CHECK_EQUAL(4, files.size()); // 3 downloaded notes + template
+    CHECK(!find_note_in_files("note2"));
+    CHECK(find_note_in_files("note4"));
+  }
+
+  TEST_FIXTURE(Fixture, download_note_update)
+  {
+    FIRST_SYNC(sync_manager1, manager1, manifest1, sync_client1, sync_ui1)
+    FIRST_SYNC(sync_manager2, manager2, manifest2, sync_client2, sync_ui2)
+
+    // update note and sync again
+    UPDATE_NOTE(manager1, "note2", "note4", "updated content");
+    sync_client1->reparse();
+    sync_manager1->perform_synchronization(sync_ui1);
+
+    // download updates
+    sync_manager2->perform_synchronization(sync_ui2);
+    get_notes_in_dir(notesdir2);
+    REQUIRE CHECK_EQUAL(4, files.size()); // 3 downloaded notes + template
+    CHECK(find_note_in_files("note1"));
+    CHECK(find_note_in_files("note3"));
+    CHECK(find_note_in_files("note4"));
+    CHECK(!find_note_in_files("note2"));
+  }
+
+  TEST_FIXTURE(Fixture, delete_note)
+  {
+    FIRST_SYNC(sync_manager1, manager1, manifest1, sync_client1, sync_ui1)
+
+    // remove note
+    auto note2 = std::dynamic_pointer_cast<test::Note>(manager1->find("note2"));
+    manager1->delete_note(note2);
+    sync_manager1->perform_synchronization(sync_ui1);
+
+    FIRST_SYNC(sync_manager2, manager2, manifest2, sync_client2, sync_ui2)
+    get_notes_in_dir(notesdir2);
+    REQUIRE CHECK_EQUAL(3, files.size()); // 2 downloaded notes + template
+    CHECK(find_note_in_files("note1"));
+    CHECK(find_note_in_files("note3"));
+    CHECK(!find_note_in_files("note2"));
+  }
+
+  TEST_FIXTURE(Fixture, note_modification_conflict)
+  {
+    FIRST_SYNC(sync_manager1, manager1, manifest1, sync_client1, sync_ui1)
+    FIRST_SYNC(sync_manager2, manager2, manifest2, sync_client2, sync_ui2)
+
+    // update note and sync again
+    UPDATE_NOTE(manager1, "note2", "note4", "updated content");
+    sync_client1->reparse();
+    sync_manager1->perform_synchronization(sync_ui1);
+
+    // update other client and sync
+    UPDATE_NOTE(manager2, "note2", "note5", "content updated");
+    sync_client2->reparse();
+    sync_manager2->perform_synchronization(sync_ui1);
+
+    // sync client1 again
+    sync_manager1->perform_synchronization(sync_ui1);
+    get_notes_in_dir(notesdir2);
+    REQUIRE CHECK_EQUAL(4, files.size()); // 3 downloaded notes + template
+    CHECK(find_note_in_files("note1"));
+    CHECK(find_note_in_files("note3"));
+    CHECK(!find_note_in_files("note2"));
+    CHECK(find_note_in_files("note4"));
+  }
+
+  TEST_FIXTURE(Fixture, conflict_with_deletion_on_server)
+  {
+    FIRST_SYNC(sync_manager1, manager1, manifest1, sync_client1, sync_ui1)
+    FIRST_SYNC(sync_manager2, manager2, manifest2, sync_client2, sync_ui2)
+
+    // remove note
+    auto note2 = std::dynamic_pointer_cast<test::Note>(manager2->find("note2"));
+    manager2->delete_note(note2);
     sync_manager2->perform_synchronization(sync_ui2);
 
-    files.clear();
-    sharp::directory_get_files_with_ext(notesdir2, ".note", files);
-    REQUIRE CHECK_EQUAL(4, files.size()); // 3 downloaded notes + template
-    CHECK(!find_note(files, "note2"));
-    CHECK(find_note(files, "note4"));
+    // update note and sync again
+    UPDATE_NOTE(manager1, "note2", "note4", "updated content");
+    sync_client1->reparse();
+    sync_manager1->perform_synchronization(sync_ui1);
+    get_notes_in_dir(notesdir1);
+    REQUIRE CHECK_EQUAL(3, files.size()); // 2 downloaded notes + template
+    CHECK(find_note_in_files("note1"));
+    CHECK(find_note_in_files("note3"));
+    CHECK(!find_note_in_files("note2"));
   }
 }
 
